@@ -20,14 +20,19 @@ class ClCommand(object):
     @classmethod
     def run(cls, comm_type, comm, **kwargs):
         fg = False
-        i = False
+        log_level = logging.DEBUG
+        log_error = True
         if 'f' in comm_type:
             fg = True
         if 'i' in comm_type:
-            i = True
+            log_level = logging.INFO
+        if 'n' in comm_type:
+            log_error = False
         try:
-            result = ClHelper.run_command(comm, fg, i)
+            result = ClHelper.run_command(comm, fg, log_level)
         except plumbum.ProcessExecutionError as e:
+            if log_error:
+                logger.error(e)
             raise exceptions.RunException(e)
 
         return result.strip() if hasattr(result, 'strip') else result
@@ -40,7 +45,9 @@ class DotDevassistantCommand(object):
     @classmethod
     def run(cls, comm_type, comm, **kwargs):
         if comm_type == 'dda_c':
-            cls._dot_devassistant_create(comm, **kwargs)
+            return cls._dot_devassistant_create(comm, **kwargs)
+        elif comm_type == 'dda_r':
+            return cls._dot_devassistant_read(comm, **kwargs)
         else:
             logger.warning('Unknown .devassistant command {0}, skipping.'.format(comm_type))
 
@@ -59,6 +66,22 @@ class DotDevassistantCommand(object):
         yaml.dump(to_write, stream=f, default_flow_style=False)
         f.close()
 
+    @classmethod
+    def _dot_devassistant_read(cls, comm, **kwargs):
+        """Don't use this directly from assistants (yet), raises uncaught exception
+        if anything goes wrong."""
+        dot_devassistant = os.path.join(os.path.abspath(os.path.expanduser(comm)), '.devassistant')
+        try:
+            with open(dot_devassistant, 'r') as stream:
+                result = yaml.load(stream)
+        except IOError as e:
+            msg = 'Couldn\'t find properly formatted .devassistant file: {0}'.format(e)
+            logger.error(msg)
+            raise exceptions.RunException(e)
+
+        result['name'] = os.path.basename(os.path.abspath(os.path.expanduser(comm)))
+        return result
+
 class GitHubCommand(object):
     @classmethod
     def matches(cls, comm_type):
@@ -101,12 +124,16 @@ class GitHubCommand(object):
         user = gh.get_user()
         try:
             if reponame in map(lambda x: x.name, user.get_repos()):
-                raise exceptions.RunException('Repository already exists on GiHub.')
+                msg = 'Repository already exists on GitHub'
+                logger.error(msg)
+                raise exceptions.RunException(msg)
             else:
                 new_repo = user.create_repo(reponame)
                 logger.info('Your new repository: {0}'.format(new_repo.html_url))
         except github.GithubException as e:
-            raise exceptions.RunException('GitHub error: {0}'.format(e))
+            msg = 'GitHub error: {0}'.format(e)
+            logger.error(msg)
+            raise exceptions.RunException(msg)
 
     @classmethod
     def _github_push(cls, **kwargs):
@@ -151,7 +178,7 @@ class GitHubCommand(object):
 
     @classmethod
     def _github_create_and_push(cls, **kwargs):
-        ClHelper.run_command('cd {0}'.format(os.path.abspath(os.path.expanduser(kwargs['name']))))
+        # we assume we're in the project directory
         logger.info('Registering your project on GitHub as {0}/{1}...'.format(cls._github_username(**kwargs),
                                                                               cls._github_reponame(**kwargs)))
         cls._github_create_repo(**kwargs)
