@@ -2,9 +2,9 @@ import logging
 import sys
 
 from devassistant.cli import argparse_generator
-from devassistant.cli import path_runner
 from devassistant import exceptions
 from devassistant import logger
+from devassistant import path_runner
 
 class CliRunner(object):
     @classmethod
@@ -21,8 +21,9 @@ class CliRunner(object):
         ch = assistant.get_subassistant_chain()
         parsed_args = argparse_generator.ArgparseGenerator.generate_argument_parser(ch).parse_args()
         path = assistant.get_selected_subassistant_path(**vars(parsed_args))
-        pr = path_runner.PathRunner(path, parsed_args)
+        pr = path_runner.PathRunner(path, vars(parsed_args))
         try:
             pr.run()
         except exceptions.ExecutionException as ex:
-            logger.logger.error(*ex.args)
+            # error is already logged, just catch it and silently exit here
+            pass
