@@ -10,21 +10,23 @@ class YamlAssistantLoader(object):
     _classes = []
 
     @classmethod
-    def get_top_level_assistants(cls):
+    def get_top_level_assistants(cls, roles=['creator', 'modifier']):
         assistants = cls.get_all_classes()
         are_subassistants = set()
         for a in assistants:
             if hasattr(a, '_subassistants'):
                 are_subassistants.update(a._subassistants)
-        return filter(lambda x: x.name not in are_subassistants, assistants)
+        top_level = filter(lambda x: x.name not in are_subassistants, assistants)
+        return filter(lambda x: x.role in roles, top_level)
 
     @classmethod
     def get_all_classes(cls):
         if not cls._classes:
             parsed_yamls = yaml_loader.YamlLoader.load_all_yamls(cls.assistants_dirs)
 
-            for y in parsed_yamls.values():
+            for s, y in parsed_yamls.items():
                 cls._classes.append(cls.class_from_yaml(y))
+                cls._classes[-1]._source_file = s
 
             for sa in cls._classes:
                 if hasattr(sa, '_subassistants'):
@@ -50,6 +52,7 @@ class YamlAssistantLoader(object):
         CustomYamlAssistant.name = name
         CustomYamlAssistant.fullname = attrs.get('fullname', '')
         CustomYamlAssistant.description = attrs.get('description', '')
+        CustomYamlAssistant.role = attrs.get('role', 'creator')
         # cli arguments
         CustomYamlAssistant.args = []
         yaml_args = attrs.get('args', {})
@@ -75,5 +78,7 @@ class YamlAssistantLoader(object):
         for k, v in attrs.items():
             if k.startswith('run') or k.startswith('dependencies'):
                 setattr(CustomYamlAssistant, '_{0}'.format(k), v)
+        CustomYamlAssistant.pre_run = attrs.get('pre_run', [])
+        CustomYamlAssistant.post_run = attrs.get('post_run', [])
 
         return CustomYamlAssistant
